@@ -1,7 +1,7 @@
 /*
  * block.h
  *
- * Copyright 2002-2016 Vesa Halttunen
+ * Copyright 2002-2017 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -101,7 +101,7 @@ public:
     void transpose(int instrument, int halfNotes, int startTrack, int startLine, int endTrack, int endLine);
 
     // Expands/shrinks a block or a part of it
-    void expandShrink(int factor, int startTrack, int startLine, int endTrack, int endLine);
+    void expandShrink(int factor, int startTrack, int startLine, int endTrack, int endLine, bool changeBlockLength);
 
     // Changes or swaps an instrument to another
     void changeInstrument(int from, int to, bool swap, int startTrack, int startLine, int endTrack, int endLine);
