@@ -1,7 +1,7 @@
 /*
  * player.cpp
  *
- * Copyright 2002-2016 Vesa Halttunen
+ * Copyright 2002-2017 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -318,7 +318,7 @@ void Player::resetPitch()
     }
 }
 
-void Player::handleCommand(QSharedPointer<TrackStatus> trackStatus, unsigned char note, unsigned char instrument, unsigned char command, unsigned char value, unsigned int *volume, int *delay, int *hold)
+void Player::handleCommand(QSharedPointer<TrackStatus> trackStatus, unsigned char note, unsigned char instrument, unsigned char command, unsigned char value, unsigned int *volume, int *delay, int *repeat, int *hold)
 {
     if (command == 0 && value == 0) {
         return;
@@ -350,6 +350,7 @@ void Player::handleCommand(QSharedPointer<TrackStatus> trackStatus, unsigned cha
         trackStatus->previousCommand = command;
     }
 
+    Track *track = song->track(trackStatus->track);
     switch (command) {
     case CommandPitchWheel:
         // Pitch wheel can be set if the MIDI channel is known
@@ -400,10 +401,12 @@ void Player::handleCommand(QSharedPointer<TrackStatus> trackStatus, unsigned cha
         *hold = value;
         break;
     case CommandRetrigger:
-        *delay = -value;
+        *delay = (value & 0xf0) >> 4;
+        *repeat = value & 0x0f;
         break;
     case CommandDelay:
         *delay = value;
+        *repeat = -1;
         break;
     case CommandVelocity:
         if (note != 0) {
@@ -479,6 +482,39 @@ void Player::handleCommand(QSharedPointer<TrackStatus> trackStatus, unsigned cha
             output->tempo(value);
         }
         break;
+    case CommandTrackVolume:
+        if (value < 0x80) {
+            if (tick == 0) {
+                track->setVolume(value);
+            }
+        } else {
+            if (tick < song->ticksPerLine() - 1) {
+                float delta = (value - 0x80 - track->volume()) / (float)song->ticksPerLine();
+                track->setVolume(track->volume() + (tick + 1) * delta);
+            } else {
+                track->setVolume(value - 0x80);
+            }
+        }
+        break;
+    case CommandInstrumentVolume: {
+        char trackInstrument = instrument != 0 ? (instrument - 1) : trackStatus->instrument;
+        Instrument *instrument = song->instrument(trackInstrument);
+        if (instrument != NULL) {
+            if (value < 0x80) {
+                if (tick == 0) {
+                    instrument->setDefaultVelocity(value);
+                }
+            } else {
+                if (tick < song->ticksPerLine() - 1) {
+                    float delta = (value - 0x80 - instrument->defaultVelocity()) / (float)song->ticksPerLine();
+                    instrument->setDefaultVelocity(track->volume() + (tick + 1) * delta);
+                } else {
+                    instrument->setDefaultVelocity(value - 0x80);
+                }
+            }
+        }
+        break;
+    }
     }
 
     // Handle MIDI controllers
@@ -503,6 +539,11 @@ void Player::handleCommand(QSharedPointer<TrackStatus> trackStatus, unsigned cha
     }
 }
 
+bool shouldPlayNote(unsigned int tick, int delay, int repeat)
+{
+    return (delay >= 0 && tick == delay) || (repeat == 0 && tick == 0) || (repeat > 0 && tick >= delay && (tick - delay) % repeat == 0);
+}
+
 void Player::run()
 {
     ExternalSync prevsyncMode = syncMode;
@@ -569,7 +610,7 @@ void Player::run()
             // The track is taken into account if the track is not muted and no tracks are soloed or the current track is soloed
             if (!song->track(track)->isMuted() && (!solo || (solo && song->track(track)->isSolo()))) {
                 unsigned int volume = 127;
-                int delay = 0, hold = -1;
+                int delay = 0, repeat = -1, hold = -1;
                 unsigned char basenote = block->note(line_, track);
                 unsigned char instrument = block->instrument(line_, track);
                 unsigned char note = basenote;
@@ -612,17 +653,19 @@ void Player::run()
 
                             switch (command) {
                             case CommandRetrigger:
-                                delay = -value;
+                                delay = (value & 0xf0) >> 4;
+                                repeat = value & 0x0f;
                                 break;
                             case CommandDelay:
                                 delay = value;
+                                repeat = -1;
                                 break;
                             }
                         }
                     }
 
                     // Stop currently playing note
-                    if ((delay >= 0 && tick == delay) || (delay < 0 && tick % (-delay) == 0)) {
+                    if (shouldPlayNote(tick, delay, repeat)) {
                         if (trackStatus->note != -1) {
                             midi_->output(trackStatus->midiInterface)->noteOff(trackStatus->midiChannel, trackStatus->note, 127);
                             trackStatus->note = -1;
@@ -635,7 +678,7 @@ void Player::run()
                     for (int commandPage = 0; commandPage < arpeggio->commandPages(); commandPage++) {
                         unsigned char command = arpeggio->command(trackStatus->line, 0, commandPage);
                         unsigned char value = arpeggio->commandValue(trackStatus->line, 0, commandPage);
-                        handleCommand(trackStatus, note, instrument, command, value, &volume, &delay, &hold);
+                        handleCommand(trackStatus, note, instrument, command, value, &volume, &delay, &repeat, &hold);
                     }
                 }
 
@@ -644,7 +687,7 @@ void Player::run()
                 for (int commandPage = 0; commandPage < commandPages; commandPage++) {
                     unsigned char command = block->command(line_, track, commandPage);
                     unsigned char value = block->commandValue(line_, track, commandPage);
-                    handleCommand(trackStatus, note, instrument, command, value, &volume, &delay, &hold);
+                    handleCommand(trackStatus, note, instrument, command, value, &volume, &delay, &repeat, &hold);
                 }
 
                 // Set the channel base note and instrument regardless of whether there's an actual note to be played right now
@@ -656,7 +699,7 @@ void Player::run()
                 }
 
                 // Is there a note to play?
-                if (note != 0 && ((delay >= 0 && tick == delay) || (delay < 0 && tick % (-delay) == 0))) {
+                if (note != 0 && shouldPlayNote(tick, delay, repeat)) {
                     Instrument *instr = NULL;
 
                     note--;
@@ -967,8 +1010,8 @@ void Player::trackStatusCreate(bool recreateAll)
     }
 
     // Set new tracks to -1
-    while (trackStatuses.count() < maxTracks) {
-        trackStatuses.append(QSharedPointer<TrackStatus>(new TrackStatus));
+    for (unsigned int track = trackStatuses.count(); track < maxTracks; track++) {
+        trackStatuses.append(QSharedPointer<TrackStatus>(new TrackStatus(track)));
     }
 }
 
@@ -1290,7 +1333,8 @@ MIDI *Player::midi() const
     return midi_;
 }
 
-Player::TrackStatus::TrackStatus()
+Player::TrackStatus::TrackStatus(unsigned int track) :
+    track(track)
 {
     reset();
 }
