@@ -1,7 +1,7 @@
 /*
  * song.cpp
  *
- * Copyright 2002-2016 Vesa Halttunen
+ * Copyright 2002-2017 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -88,7 +88,7 @@ Song::~Song()
 
 void Song::init()
 {
-    name_ = "Untitled";
+    name_ = tr("Untitled");
     tempo_ = 120;
     ticksPerLine_ = 6;
     masterVolume_ = 127;
@@ -474,9 +474,9 @@ void Song::checkMaxTracks()
     }
 
     if (oldMax < max) {
-        for (int track = oldMax; track < max; track++) {
-            // Give a descriptive name for the new track
-            addTrack(QString("Track %1").arg(track + 1));
+        while (tracks.count() < max) {
+            // Give a descriptive name for each new track
+            addTrack(-1, tr("Track %1").arg(tracks.count() + 1));
         }
     } else if (oldMax > max) {
         // Tracks removed: free track datas
@@ -493,7 +493,7 @@ void Song::checkMaxTracks()
 void Song::checkInstrument(int instrument)
 {
     while (instrument >= instruments_.count()) {
-        Instrument *instrument = new Instrument("Unnamed");
+        Instrument *instrument = new Instrument(tr("Unnamed"));
         connectInstrumentSignals(instrument);
         instruments_.append(instrument);
     }
@@ -506,10 +506,10 @@ void Song::transpose(int instrument, int halfNotes)
     }
 }
 
-void Song::expandShrink(int factor)
+void Song::expandShrink(int factor, bool changeBlockLength)
 {
     for (int block = 0; block < blocks_.count(); block++) {
-        blocks_[block]->expandShrink(factor, 0, 0, blocks_[block]->tracks() - 1, blocks_[block]->length() - 1);
+        blocks_[block]->expandShrink(factor, 0, 0, blocks_[block]->tracks() - 1, blocks_[block]->length() - 1, changeBlockLength);
     }
 }
 
@@ -520,6 +520,40 @@ void Song::changeInstrument(int from, int to, bool swap)
     }
 }
 
+void Song::insertTrack(int track)
+{
+    addTrack(track, tr("Track %1").arg(track + 1));
+    for (int block = 0; block < blocks_.count(); block++) {
+        disconnect(blocks_[block], SIGNAL(tracksChanged(int)), this, SLOT(checkMaxTracks()));
+        blocks_[block]->insertTrack(track);
+        connect(blocks_[block], SIGNAL(tracksChanged(int)), this, SLOT(checkMaxTracks()));
+    }
+
+    // Maximum number of tracks will have changed since a track has been added to every block
+    emit maxTracksChanged(tracks.count());
+
+    emit tracksChanged();
+}
+
+void Song::deleteTrack(int track)
+{
+    if (maxTracks() > 1) {
+        for (int block = 0; block < blocks_.count(); block++) {
+            disconnect(blocks_[block], SIGNAL(tracksChanged(int)), this, SLOT(checkMaxTracks()));
+            blocks_[block]->deleteTrack(track);
+            connect(blocks_[block], SIGNAL(tracksChanged(int)), this, SLOT(checkMaxTracks()));
+        }
+        Track *trackToBeDeleted = tracks.takeAt(track);
+
+        // Maximum number of tracks will have changed since a track has been deleted from every block
+        emit maxTracksChanged(tracks.count());
+
+        emit tracksChanged();
+
+        delete trackToBeDeleted;
+    }
+}
+
 bool Song::parse(QDomElement element)
 {
     if (element.tagName() == "song") {
@@ -894,14 +928,17 @@ void Song::unlock()
     mutex.unlock();
 }
 
-void Song::addTrack(const QString &name)
+void Song::addTrack(int index, const QString &name)
 {
     Track *track = new Track(name);
     connect(track, SIGNAL(mutedChanged(bool)), this, SIGNAL(trackMutedOrSoloed()));
     connect(track, SIGNAL(soloChanged(bool)), this, SIGNAL(trackMutedOrSoloed()));
     connect(track, SIGNAL(nameChanged(QString)), this, SIGNAL(trackNameChanged()));
     connect(track, SIGNAL(volumeChanged(int)), this, SIGNAL(trackVolumeChanged()));
-    tracks.append(track);
+    if (index < 0 || index > tracks.count()) {
+        index = tracks.count();
+    }
+    tracks.insert(index, track);
 }
 
 void Song::connectBlockSignals(Block *block)
